@@ -112,6 +112,23 @@ class MultiQubitSystem:
                 prod = np.kron(prod, self.dict_proj[povm[j]])
         return prod
 
+
+    def Sorted_Proj_Strings(self):
+        POVM_list = ['h', 'v', 'd', 'a', 'r', 'l']
+        repeated = np.repeat(POVM_list, 1000000)
+        np.random.shuffle(repeated)
+        repeated_shuffled = repeated.reshape(-1, self.qubit_size)
+        sorted_proj = np.unique(repeated_shuffled, axis=0).reshape(-1,self.qubit_size)
+
+        assert len(sorted_proj) == 6 ** self.qubit_size, 'Increase the sampling in the repeated povm list'
+        return sorted_proj
+
+    def Alphabetical_Projectors(self):
+        sorted_proj = self.Sorted_Proj_Strings()
+        Proj = list(map(self.Kron_Povm, sorted_proj))
+        Proj = np.array(Proj).reshape(-1, 2 ** self.qubit_size, 2 ** self.qubit_size)
+        return Proj
+
     def NISQ_Projectors(self):
         ibmq_proj = Gen_Basis_Order(qs=self._qs).Basis_Order()
         # print(ibmq_proj)
@@ -134,21 +151,55 @@ class MultiQubitSystem:
 
 class Ideal_Measurements:
 
-    def __init__(self, qs=2):
+    def __init__(self, qs=2, projectors_kind='nisq'): # projectors_kind == nisq, alphabetical, general
         self._qs = qs
-        if not os.path.exists(f'./projectors/projectors_array_qs_{self._qs}.pickle'):
-            print('| To accelerate the simulation, a new projector folder is created.')
-            try:
-                os.mkdir('projectors')
-            except:
-                print('|Found projectors folder! Appending some files ...')
-            mqs = MultiQubitSystem(qubit_size=self._qs)
-            proj = mqs.NISQ_Projectors()
-            with open(f'./projectors/projectors_array_qs_{self._qs}.pickle', 'wb') as f:
-                pkl.dump(proj, f, -1)
-            self.projectors = proj
+        if projectors_kind == 'nisq':
+            if not os.path.exists(f'./projectors/projectors_array_qs_{self._qs}_nisq.pickle'):
+                print('| To accelerate the simulation, a new nisq projector folder is created.')
+                try:
+                    os.mkdir('projectors')
+                except:
+                    print('|Found projectors folder! Appending some files ...')
+                mqs = MultiQubitSystem(qubit_size=self._qs)
+                proj = mqs.NISQ_Projectors()
+                with open(f'./projectors/projectors_array_qs_{self._qs}_nisq.pickle', 'wb') as f:
+                    pkl.dump(proj, f, -1)
+                self.projectors = proj
+            else:
+                self.projectors = pd.read_pickle(f'./projectors/projectors_array_qs_{self._qs}_nisq.pickle')
+        elif projectors_kind == 'alphabetical':
+            if not os.path.exists(f'./projectors/projectors_array_qs_{self._qs}_alphabetical.pickle'):
+                print('| To accelerate the simulation, a new alphabetical projector folder is created.')
+                try:
+                    os.mkdir('projectors')
+                except:
+                    print('|Found alphabetical projectors folder! Appending some files ...')
+                mqs = MultiQubitSystem(qubit_size=self._qs)
+                proj = mqs.Alphabetical_Projectors()
+                with open(f'./projectors/projectors_array_qs_{self._qs}_alphabetical.pickle', 'wb') as f:
+                    pkl.dump(proj, f, -1)
+                self.projectors = proj
+            else:
+                self.projectors = pd.read_pickle(f'./projectors/projectors_array_qs_{self._qs}_alphabetical.pickle')
+
+        elif projectors_kind == 'general':
+            if not os.path.exists(f'./projectors/projectors_array_qs_{self._qs}_general.pickle'):
+                print('| To accelerate the simulation, a new general projector folder is created.')
+                try:
+                    os.mkdir('projectors')
+                except:
+                    print('|Found general projectors folder! Appending some files ...')
+                mqs = MultiQubitSystem(qubit_size=self._qs)
+                proj = mqs.General_Scheme_Projectors()
+                with open(f'./projectors/projectors_array_qs_{self._qs}_general.pickle', 'wb') as f:
+                    pkl.dump(proj, f, -1)
+                self.projectors = proj
+            else:
+                self.projectors = pd.read_pickle(f'./projectors/projectors_array_qs_{self._qs}_general.pickle')
         else:
-            self.projectors = pd.read_pickle(f'./projectors/projectors_array_qs_{self._qs}.pickle')
+            print(f'| Given projectors config {projectors_kind} can not be found!')
+            os.system('exit')
+
 
     def get_tau_cholesky(self, density_matrix):
         try:
